package bioinformatica.programmazione.esami.luglio.terzaparte;

/**
 * Classe che rappresenta una elezione politica. Raggruppa i partiti in
 * coalizioni.
 * 
 * @author progBioinfo
 * 
 */
public class Elezione {

	/**
	 * Le coalizioni registrate per questa elezione. Se un partito si presenta
	 * da solo, starà in una coalizione in cui è presente solo esso stesso.
	 */
	private Coalizione[] coalizioni;

	/**
	 * Crea una nuova istanza di elezione senza coalizioni.
	 */
	public Elezione() {
		coalizioni = new Coalizione[0];
	}

	/**
	 * Registra il partito indicato come un partecipante all'elezione, dentro
	 * una coalizione di cui fa parte solo esso stesso. Se una coalizione con lo
	 * stesso nome è già presente, genera una
	 * <code>CoalizioneGiaPresenteException</code>. Se il partito è già
	 * registrato, genera una <code>PartitoGiaRegistratoException</code>.
	 * 
	 * @param partito
	 *            il partito da registrare.
	 * @throws PartitoGiaRegistratoException
	 *             se <code>partito</code> è già registrato con qualche altra
	 *             coalizione.
	 * @throws CoalizioneGiaPresenteException
	 *             se è già presente una coalizione con lo stesso nome.
	 */
	public void registra(Partito partito) throws PartitoGiaRegistratoException,
			CoalizioneGiaPresenteException {
		registra(new Coalizione(partito.getNome(), partito));
	}

	/**
	 * Registra la coalizione indicata come partecipante all'elezione. Se esiste
	 * già una coalizione uguale (con lo stesso nome), genera una
	 * <code>CoalizioneGiaPresenteException</code>. Se uno dei partiti da
	 * mettere in coalizione appartiene ad una coalizione precedentemente
	 * creata, genera una <code>PartitoGiaRegistratoException</code>.
	 * 
	 * @param coalizione
	 *            la coalizione da registrare.
	 * @throws PartitoGiaRegistratoException
	 *             se uno dei partiti da registrare appare già in qualche
	 *             coalizione già registrata.
	 * @throws CoalizioneGiaPresenteException
	 *             se esiste già una coalizione con lo stesso nome.
	 */
	public void registra(Coalizione coalizione)
			throws PartitoGiaRegistratoException,
			CoalizioneGiaPresenteException {

		if (coalizione == null)
			return;

		// verifichiamo che non sia gia' presente una coalizione con lo stesso
		// nome
		for (Coalizione c : coalizioni)
			if (c.equals(coalizione))
				throw new CoalizioneGiaPresenteException(coalizione.getNome());

		// verifichiamo che i partiti della nuova coalizione non siano gia'
		// presenti in altre coalizioni
		for (Partito p : coalizione.getPartiti())
			for (Coalizione c : coalizioni)
				if (c.include(p))
					throw new PartitoGiaRegistratoException(p.getNome());

		// estendiamo il gruppo di coalizioni
		Coalizione[] newCoalizioni = new Coalizione[coalizioni.length + 1];
		int i = 0;
		for (Coalizione c : coalizioni) {
			newCoalizioni[i] = c;
			i++;
		}
		// aggiungiamo la nuova coalizione
		newCoalizioni[i] = coalizione;
		coalizioni = newCoalizioni;
	}

	/**
	 * Registra un nuovo voto per il partito indicato (e quindi anche per la
	 * coalizione di cui esso fa parte). Se il partito non è registrato
	 * all'elezione, genera una <code>PartitoMaiRegistratoException</code>.
	 * 
	 * @param partito
	 *            il partito a cui assegnare un voto.
	 * @throws PartitoMaiRegistratoException
	 *             se il partito non appare in nessuna coalizione.
	 */
	public void registraVoto(Partito partito)
			throws PartitoMaiRegistratoException {
		if (partito == null)
			return;
		for (Coalizione c : coalizioni) {
			for (Partito p : c.getPartiti())
				if (partito.equals(p)) {
					p.aggiungiVoto();
					return;
				}
		}
		throw new PartitoMaiRegistratoException(partito.getNome());
	}

	@Override
	public String toString() {
		String risultato = "";

		for (Coalizione c : coalizioni) {
			risultato += "coalizione \"" + c.getNome() + "\"\n";
			for (Partito p : c.getPartiti())
				risultato += "\t" + p.getNome() + ", voti: " + p.getVoti()
						+ "\n";
			risultato += "\tvoti di coalizione: " + c.getVoti() + "\n";
		}
		return risultato;
	}

}